package
{
	import com.greensock.TweenMax;
	import com.greensock.easing.Quart;
	
	import flash.display.Sprite;
	import flash.display.Stage;
	import flash.display.StageAlign;
	import flash.display.StageOrientation;
	import flash.display.StageScaleMode;
	import flash.events.Event;
	import flash.events.EventDispatcher;
	import flash.filesystem.File;
	
	import uk.co.creativewax.display.DeviceOrientation;
	import uk.co.creativewax.pages.PageData;
	import uk.co.creativewax.pages.Pages;
	import uk.co.creativewax.saving.Saving;
	import uk.co.creativewax.utils.Utils;
	import uk.co.creativewax.views.CaptureView;
	import uk.co.creativewax.views.LoaderView;
	import uk.co.creativewax.views.ViewerView;

	public class Application extends EventDispatcher
	{
		public static const SAVE_APP_DATA:String	= "SAVE_APP_DATA";
		
		private static var instance:Application;
		
		private var _holder:Sprite					= new Sprite();						// holding sprite for edit and viewer views
		private var _capture:CaptureView			= new CaptureView();				// the capture view
		private var _viewer:ViewerView				= new ViewerView();					// the viewer view
		private var _pages:Pages					= Pages.getInstance();				// get an instance of Pages in the app
		private var _saving:Saving					= Saving.getInstance();				// get an instance of the Saving Singleton Class
		private var _loader:LoaderView				= LoaderView.getInstance();			// get an instance of the Loader View Singleton Class
		private var _orientation:DeviceOrientation	= DeviceOrientation.getInstance();	// get an instance of the Device Orientation Singleton Class
		
		public static function getInstance():Application
		{
			if (instance == null) instance = new Application(new SingletonBlocker());
			return instance;
		}
		
		public function Application(key:SingletonBlocker)
		{
			if (key == null) throw new Error("Error: Instantiation failed: Use Application.getInstance() instead of new.");
		}
		
		
		
		// ______________________________________________
		// PUBLIC
		
		public function setup(stage:Stage):void
		{
			// setup stage
			stage.scaleMode		= StageScaleMode.NO_SCALE;
			stage.align			= StageAlign.TOP_LEFT;
			
			
			// load the current app data
			// Resolve the path for saving the app data, this should be in the app storage directory, per iPhone guidelines.
			var file:File		= File.applicationStorageDirectory.resolvePath("data.xml");
			
			// Check if it exists
			if (file.exists)
			{
				// Lets load the app data
				Utils.loadXML(file.url, onAppDataLoaded);
				
				// Show a Loader whilst the data loads
				_loader.show("Loading Journal Data");
			}
			else
			{
				// Set a default page for starting the app
				_pages.addPage();
				
				// Tell the Capture View Class about the pages data
				_capture.showCurrentPageData();
			}
			
			
			// setup listeners
			Utils.weakEvent(this, SAVE_APP_DATA, saveAppData);
			Utils.weakEvent(_capture, SAVE_APP_DATA, saveAppData);
			Utils.weakEvent(_pages, Pages.PAGES_SETUP, onPagesSetup);
			Utils.weakEvent(_pages, Pages.CURRENT_PAGE_UPDATED, onPagesUpdated);
			Utils.weakEvent(_orientation, DeviceOrientation.ORIENTATION_CHANGE, onOrientationChanged);
			
			// setup classes
			_capture.setup();
			_viewer.setup();
			_orientation.setup(stage);
			
			// setup display list
			_holder.addChild(_capture);
			_holder.addChild(_viewer);
			stage.addChild(_holder);
			stage.addChild(_loader);
		}
		
		
		
		
		// ______________________________________________
		// PROTECTED
		
		
		
		
		// ______________________________________________
		// PRIVATE
		
		// Setup the Pages now the app data has been loaded
		private function onAppDataLoaded(e:Event):void
		{
			_pages.setFromXML(XML(e.target.data));	// Set the pages data from the XML
		}
		
		// Invoked when the Pages have been setup on app start
		private function onPagesSetup(e:Event):void
		{
			// Hide the Loader
			_loader.hide();
			
			// Tell the Capture View Class about the pages data
			_capture.showCurrentPageData();
		}
		
		// Invoked when the current page number has been updated in the Pages Class
		private function onPagesUpdated(e:Event):void
		{
			// Tell the Capture View Class about the pages data
			_capture.showCurrentPageData();
		}
		
		// save the latest page data so that its persistent on the next launch
		private function saveAppData(e:Event):void
		{
			_saving.saveAppData(_pages.getPages());
		}
		
		// Invoked when the Device orientation has changed
		private function onOrientationChanged(e:Event):void
		{
			checkShowEditOrView(_orientation.currentOrientation);
		}
		
		// Check if we are showing the editer or viewer
		public function checkShowEditOrView(currentOrientation:String):void
		{
			switch(currentOrientation)
			{
				case StageOrientation.ROTATED_LEFT:
				case StageOrientation.ROTATED_RIGHT:
				{
					// Show the viewer view and tell it to check its content
					TweenMax.to(_holder, 1, { y: 480, ease: Quart.easeOut });
					_viewer.checkPageContent();
					break;
				}
					
				default:
				{
					// Show the edit view
					TweenMax.to(_holder, 1, { y: 0, ease: Quart.easeOut });
					break;
				}
			}
		}
		
		
		
		
		// ______________________________________________
		// GETTERS & SETTERS
		
	}
}

internal class SingletonBlocker {}