package uk.co.creativewax.display
{
	import flash.display.Stage;
	import flash.events.AccelerometerEvent;
	import flash.events.Event;
	import flash.events.EventDispatcher;
	import flash.sensors.Accelerometer;

	public class DeviceOrientation extends EventDispatcher
	{
		public static const ORIENTATION_CHANGE:String	= "ORIENTATION_CHANGE";
		
		private static var instance:DeviceOrientation;
		
		private var _stage:Stage;						// Stage reference for orientation checking
		private var _accelerometer:Accelerometer;		// Accelerometer reference
		private var _currentOrientation:String;			// String for local orientation checking
		
		public static function getInstance():DeviceOrientation
		{
			if (instance == null) instance = new DeviceOrientation(new SingletonBlocker());
			return instance;
		}
		
		public function DeviceOrientation(key:SingletonBlocker)
		{
			if (key == null) throw new Error("Error: Instantiation failed: Use DeviceOrientation.getInstance() instead of new.");
		}
		
		
		
		// ______________________________________________
		// PUBLIC
		
		// Setup the accelerometer to check the Device Orientation
		// we pass in the stage reference so we can do a check against the stages current orientation
		// as and when the Device orientation changes
		public function setup(stage:Stage):void
		{
			// Update stage reference
			_stage = stage;
			
			// Set up accelerometer for checking the orientation changes on the device
			_accelerometer	= new Accelerometer();
			_accelerometer.setRequestedUpdateInterval(150);
			_accelerometer.addEventListener(AccelerometerEvent.UPDATE, accelerometerUpdate);
			
			// Update the current orientation reference and tell the app about the Devices current orientation
			accelerometerUpdate();
		}
		
		
		
		
		// ______________________________________________
		// PROTECTED
		
		
		
		
		// ______________________________________________
		// PRIVATE
		
		// Invoked everytime the accelerometer timer ticks over
		public function accelerometerUpdate(e:AccelerometerEvent=null):void
		{
			// Check to see if the orientation changes at all, if so we need to tell the app
			if (!e || _stage.deviceOrientation != _currentOrientation)
			{
				_currentOrientation	= _stage.deviceOrientation;			// Update local orientation checker
				this.dispatchEvent(new Event(ORIENTATION_CHANGE));		// Tell app about the orientation change
			}
		}
		
		
		
		
		// ______________________________________________
		// GETTERS & SETTERS
		
		// Return the current orientation of the device
		public function get currentOrientation():String		{ return _currentOrientation; }
	}
}

internal class SingletonBlocker {}