package uk.co.creativewax.display
{
	import flash.desktop.NativeApplication;
	import flash.display.Bitmap;
	import flash.display.Loader;
	import flash.display.Sprite;
	import flash.display.StageOrientation;
	import flash.events.ErrorEvent;
	import flash.events.Event;
	import flash.events.IOErrorEvent;
	import flash.events.MediaEvent;
	import flash.events.MouseEvent;
	import flash.media.CameraUI;
	import flash.media.MediaPromise;
	import flash.media.MediaType;
	
	import uk.co.creativewax.utils.Utils;
	
	public class ImageCapture extends Sprite
	{
		public static const IMAGE_LOADING:String	= "IMAGE_LOADING";
		public static const IMAGE_LOADED:String		= "IMAGE_LOADED";
		
		private var _cam:CameraUI;				// Device CameraUI
		private var _imageLoader:Loader;		// Standard Loader Class for loading content from CameraUI
		private var _currentOrientation:String;	// String orientation reference to the Image captured
		private var _bmp:Bitmap;				// Current captured bitmap
		
		private var _orientation:DeviceOrientation	= DeviceOrientation.getInstance();	// get an instance of the Device Orientation Singleton Class
		
		public function ImageCapture()
		{
			super();
		}
		
		
		
		// ______________________________________________
		// PUBLIC
		
		/**
		 * Create a new instance of the Native CameraUI on the device
		 * @param e	: MouseEvent is optional if invoked from a button press
		 */
		public function captureImage(e:MouseEvent=null):void
		{
			// Check if the CameraUI is supported
			if (CameraUI.isSupported)
			{
				_cam	= new CameraUI();
				
				// Add listeners and launch the CameraUI
				Utils.weakEvent(_cam, MediaEvent.COMPLETE, onMediaComplete);
				Utils.weakEvent(_cam, MediaEvent.SELECT, onMediaComplete);
				Utils.weakEvent(_cam, Event.CANCEL, onCaptureCanceled);
				Utils.weakEvent(_cam, ErrorEvent.ERROR, onCameraError);
				
				_cam.launch(MediaType.IMAGE);
			}
		}
		
		// Once the image has been loaded and used we can remove unwanted bits from memory
		public function destroy():void
		{
			_cam			= null;
			_imageLoader	= null;
			_bmp			= null;
		}
		
		
		
		
		
		// ______________________________________________
		// PROTECTED
		
		
		
		
		// ______________________________________________
		// PRIVATE
		
		// Media is complete and ready to be loaded
		private function onMediaComplete(e:MediaEvent):void
		{
			// Update the reference to what the Device Orientation was
			// at the point of capturing this image
			_currentOrientation	= _orientation.currentOrientation;
			
			// Get the media promise passed through frm the media event
			var imagePromise:MediaPromise = e.data;
			
			// Create a new image loader for loading the image from the CameraUI
			_imageLoader = new Loader();
			
			// Tell the app that the image is loading
			this.dispatchEvent(new Event(IMAGE_LOADING));
			
			// Check if the promise is Asynchronous or not
			if (imagePromise.isAsync)
			{
				// Asynchronous media promise
				Utils.weakEvent(_imageLoader.contentLoaderInfo, Event.COMPLETE, onAsyncImageLoaded);
				Utils.weakEvent(_imageLoader, IOErrorEvent.IO_ERROR, onCameraError);
				
				_imageLoader.loadFilePromise(imagePromise);
			}
			else
			{
				// Synchronous media promise
				_imageLoader.loadFilePromise(imagePromise);
				showMedia(_imageLoader);
			}
		}
		
		// Media is loaded and ready to be shown
		private function showMedia(imageLoader:Loader):void
		{
			// Set imageloader and bitmap reference
			_imageLoader	= imageLoader;
			_bmp			= Bitmap(imageLoader.content);
			
			// Make sure the image is the right size, rotation and crop for the Device
			checkImageState();
			
			// Tell the app that the image is ready to be used
			this.dispatchEvent(new Event(IMAGE_LOADED));
		}
		
		// Make sure the image is the right size, rotation and crop for the Device
		private function checkImageState():void
		{
			// Because the Device can return back the image in certain sizes and rotations based on the
			// device orientation, we need to detect the current orientation and use this to size
			// and rotate the image optimally
			
			// first put the image into a temp holding sprite
			var s:Sprite	= new Sprite();
			s.addChild(_bmp);
			
			// detect the orientation and use this to rotate the image accordingly
			if (currentOrientation == StageOrientation.DEFAULT)				_bmp.transform.matrix.rotate(90);
			else if (currentOrientation == StageOrientation.ROTATED_RIGHT)	_bmp.transform.matrix.rotate(180);
			else if (currentOrientation == StageOrientation.UPSIDE_DOWN)	_bmp.transform.matrix.rotate(-90);
			
			// double check the image has been rotated right, as sometimes a slight lag can take place
			// with the device not updating the orientation in time
			if (s.height > s.width) _bmp.transform.matrix.rotate(90);
			
			// resize the content to the maximum size we will show it in the app so we are saving as
			// much memory and space on the device as possible and re-capture the bitmap at its new size
			_bmp	= Utils.createBitmapToSize(s, 380, 280);
			
			// remove the temp sprite
			s = null;
		}
		
		
		private function onAsyncImageLoaded(e:Event):void		{ showMedia(_imageLoader); }						// Show the image loaded
		private function onCaptureCanceled(e:Event):void		{ NativeApplication.nativeApplication.exit(); }		// Cancelled, Exit CameraUI
		private function onCameraError(error:ErrorEvent):void	{ NativeApplication.nativeApplication.exit(); }		// Error, Exit CameraUI
		
		
		
		// ______________________________________________
		// GETTERS & SETTERS
		
		// Return the bitmap that was loaded from the CameraUI
		public function get image():Bitmap		{ return _bmp ? _bmp : null; }
		
		// Return the orientation at the point of capturing the image
		public function get currentOrientation():String		{ return _currentOrientation; }
	}
}