package uk.co.creativewax.pages
{
	import flash.events.Event;
	import flash.events.EventDispatcher;
	
	import uk.co.creativewax.utils.Utils;

	public class Pages extends EventDispatcher
	{
		public static const PAGES_SETUP:String			= "PAGES_SETUP";
		public static const CURRENT_PAGE_UPDATED:String	= "CURRENT_PAGE_UPDATED";
		
		private static var instance:Pages;
		
		private var _pages:Vector.<PageData>	= new Vector.<PageData>();	// Vector holding all pages
		private var _total:uint					= 0;						// Total number of pages
		private var _currentPage:int			= 0;						// Current page being viewed
		
		
		public static function getInstance():Pages
		{
			if (instance == null) instance = new Pages(new SingletonBlocker());
			return instance;
		}
		
		public function Pages(key:SingletonBlocker)
		{
			if (key == null) throw new Error("Error: Instantiation failed: Use Pages.getInstance() instead of new.");
		}
		
		
		
		// ______________________________________________
		// PUBLIC
		
		// Setup page from XML data
		public function setFromXML(xml:XML):void
		{
			// Read throguh the XML loaded
			var xmlList:XMLList			= xml.page;						// The XML page nodes
			var total:uint				= xmlList.length();				// Total number of nodes
			_pages						= new Vector.<PageData>();		// Vector that will hold all of the Pages page data
			
			// Loop through and populate the vector with the page data
			for (var i:uint=0; i<total; i++)
			{
				// Get the data from the page node
				var data:XML			= xmlList[i];
				var ID:String			= data.@id;
				var hasImage:Boolean	= Utils.parseStringBoolean(data.@image);
				var hasAudio:Boolean	= Utils.parseStringBoolean(data.@audio);
				var hasvideo:Boolean	= Utils.parseStringBoolean(data.@video);
				
				// Add to the pages vector
				_pages.push(new PageData(ID, hasImage, hasAudio, hasvideo));
			}
			
			// tell app that pages data is now loaded and ready to be used
			this.dispatchEvent(new Event(PAGES_SETUP));
		}
		
		// Add a single set of PageData to the pages
		public function addPage():void
		{
			// Create a new PageData Class and Create a new DateStamp to make its ID unique
			var pd:PageData	= new PageData(Utils.getDateString());
			
			// Add to Vector and increment the total
			_pages.push(pd);
			_total++;
		}
		
		// Replace all of the pages data with the Vector passed
		public function addPages(pages:Vector.<PageData>):void
		{
			_pages	= pages;
			_total	= pages.length;
		}
		
		// Return Current / Previous / Next PageData
		public function getCurrentPage():PageData	{ return getPage(0); }
		public function getPreviousPage():PageData	{ return getPage(-1); }
		public function getNextPage():PageData		{ return getPage(1); }
		
		// Return a page at a certain point in the page
		public function getPage(i:int):PageData
		{
			// Make sure the page request is within the limits of the pages
			var current:int	= _currentPage+i;
			var prev:int	= _currentPage;
			
			if (current <= 0)
			{
				current = 0;			// If we go below the base page then set current to the base position
			}
			else if (current >= totalPages)
			{
				addPage();				// Else if we are already at the last page then simply add a new page and move to it
				current = totalPages-1;	// Cap the current count
			}
			
			// Update current reference
			_currentPage = current;
			
			// Check that the page we are moving to is different from the page we are on
			if (prev != current) this.dispatchEvent(new Event(CURRENT_PAGE_UPDATED));
			
			// return page data
			return _pages[_currentPage];
		}
		
		// Return the latest pages in the app
		public function getPages():Vector.<PageData>
		{
			return _pages;
		}
		
		
		
		
		
		// ______________________________________________
		// PROTECTED
		
		
		
		
		// ______________________________________________
		// PRIVATE
		
		
		
		
		// ______________________________________________
		// GETTERS & SETTERS
		
		public function get currentPage():uint		{ return _currentPage; }
		public function get totalPages():uint		{ return _total; }
		
		public function get ID():String					{ return getCurrentPage().ID; }
		public function get hasImage():Boolean			{ return getCurrentPage().hasImage; }
		public function set hasImage(b:Boolean):void	{ getCurrentPage().hasImage = b; }
		public function get hasAudio():Boolean			{ return getCurrentPage().hasAudio; }
		public function set hasAudio(b:Boolean):void	{ getCurrentPage().hasAudio = b; }
		public function get hasVideo():Boolean			{ return getCurrentPage().hasVideo; }
		public function set hasVideo(b:Boolean):void	{ getCurrentPage().hasVideo = b; }
	}
}

internal class SingletonBlocker {}