package uk.co.creativewax.saving
{
	import by.blooddy.crypto.image.PNG24Encoder;
	
	import flash.display.BitmapData;
	import flash.events.Event;
	import flash.events.EventDispatcher;
	import flash.filesystem.File;
	import flash.filesystem.FileMode;
	import flash.filesystem.FileStream;
	import flash.utils.ByteArray;
	import uk.co.creativewax.pages.PageData;

	public class Saving extends EventDispatcher
	{
		public static const SAVED_APP_DATA:String	= "SAVED_APP_DATA";
		public static const SAVED_IMAGE:String		= "SAVED_IMAGE";
		public static const SAVED_AUDIO:String		= "SAVED_AUDIO";
		
		private static var instance:Saving;
		
		
		public static function getInstance():Saving
		{
			if (instance == null) instance = new Saving(new SingletonBlocker());
			return instance;
		}
		
		public function Saving(key:SingletonBlocker)
		{
			if (key == null) throw new Error("Error: Instantiation failed: Use Saving.getInstance() instead of new.");
		}
		
		
		
		// ______________________________________________
		// PUBLIC
		
		/**
		 * Save the main app data in an XML format
		 * @param pages	: Vector holding simple PageData classes to describe the content on each page in the app
		 */
		public function saveAppData(pages:Vector.<PageData>):void
		{
			// Create XML content
			var xml:String	= "<?xml version='1.0' encoding='UTF-8'?>";
			xml				+= "<xml>";
			
			// Loop through all of the pages we have in the app and create an entry in the XML for them
			for (var i:uint=0; i<pages.length; i++)
			{
				// Get the current page data
				var pd:PageData	= pages[i];
				
				// Write an entry into the XML for this page
				xml += "<page id='"+pd.ID+"' image='"+pd.hasImage+"' audio='"+pd.hasAudio+"' video='"+pd.hasVideo+"' />";
			}
			
			// Close the XML content tag
			xml			+= "</xml>";
			
			// Save XML file
			saveXML(xml);
			
			// Tell listeners that the app data is now saved
			this.dispatchEvent(new Event(SAVED_APP_DATA));
		}
		
		/**
		 * Save an image to the device
		 * @param bmd		: The raw bitmapdata of the image we are saving
		 * @param fileName	: The file name of the image we are saving
		 */
		public function saveImage(bmd:BitmapData, fileName:String):void
		{
			// Compress the raw bitmapdata of the image into a PNG format
			var bytes:ByteArray	= PNG24Encoder.encode(bmd);
			
			// Resolve the path for saving the image, this should be in the app storage directory, per iPhone guidelines.
			// Also make sure we append the png suffix so the image is saved correctly
			var file:File		= File.applicationStorageDirectory.resolvePath(fileName+".png");
			
			var fs:FileStream	= getFileStream(file, true, false);	// Get the filestream
			fs.writeBytes(bytes);									// Write the bytes to the stream
			fs.close();												// Make sure we close the stream
			
			// Tell listeners that the image is now saved
			this.dispatchEvent(new Event(SAVED_IMAGE));
		}
		
		/**
		 * Save a recording to the device
		 * @param audio		: The raw bytearray of the audio we are saving
		 * @param fileName	: The file name of the audio we are saving
		 */
		public function saveAudio(audio:ByteArray, fileName:String):void
		{
			// Resolve the path for saving the image, this should be in the app storage directory, per iPhone guidelines.
			// Also make sure we append the png suffix so the image is saved correctly
			var file:File		= File.applicationStorageDirectory.resolvePath(fileName+".wav");
			trace("saveAudio", file.url);
			var fs:FileStream	= getFileStream(file, true, false);	// Get the filestream
			fs.writeBytes(audio);									// Write the bytes to the stream
			fs.close();												// Make sure we close the stream
			
			// Tell listeners that the audio is now saved
			this.dispatchEvent(new Event(SAVED_AUDIO));
		}
		
		
		
		
		// ______________________________________________
		// PROTECTED
		
		
		
		
		// ______________________________________________
		// PRIVATE
		
		/**
		 * Save the main app data in an XML format
		 * @param xml	: String of xml content to save
		 */
		private function saveXML(xml:String):void
		{
			// Resolve the path for saving the app data, this should be in the app storage directory, per iPhone guidelines.
			var file:File		= File.applicationStorageDirectory.resolvePath("data.xml");
			
			// Get the filestream
			var fs:FileStream	= getFileStream(file, true, false);
			
			// Run a regular expression over the XML content to resolve the line endings,
			// not always needed but helps solve unwanted problems
			xml	= xml.replace(/\r/g, File.lineEnding);
			
			// Write the file and close the stream
			fs.writeUTFBytes(xml);
			fs.close();
		}
		
		/**
		 * Get a FileStream for reading or writing the file. 
		 * @param file	: The filepath to the data file ( this should live in the app storage directory, per iPhone guidelines )
		 * @param write	: If true, we will write to the file. If false, we will read.
		 * @param sync	: If true, we do synchronous writes. If false, asynchronous.
		 * @return A FileStream instance we can read or write with. Don't forget to close it!
		 */
		private function getFileStream(file:File, write:Boolean=false, sync:Boolean=true):FileStream
		{
			
			// Check the file exists if we are reading
			if (!write && !file.exists) return null;
			
			// Try creating and opening the stream.
			var fs:FileStream = new FileStream();
			
			try
			{
				if (write && !sync)	fs.openAsync(file, FileMode.WRITE);						// If we are writing asynchronously, openAsync.
				else				fs.open(file, write ? FileMode.WRITE : FileMode.READ);	// For synchronous write, or all reads, open synchronously.
			}
			catch(e:Error)
			{
				// On error, simply return null.
				return null;
			}
			
			return fs;
		}
		
		
		
		
		// ______________________________________________
		// GETTERS & SETTERS
		
	}
}

internal class SingletonBlocker {}