package uk.co.creativewax.sound
{
	import flash.display.Sprite;
	import flash.events.Event;
	
	import org.bytearray.micrecorder.MicRecorder;
	import org.bytearray.micrecorder.encoder.WaveEncoder;
	import org.bytearray.micrecorder.events.RecordingEvent;
	
	import uk.co.creativewax.pages.Pages;
	import uk.co.creativewax.saving.Saving;
	import uk.co.creativewax.utils.Utils;
	import uk.co.creativewax.views.LoaderView;
	
	public class AudioCapture extends Sprite
	{
		public static const AUDIO_RECORDED:String	= "AUDIO_RECORDED";
		
		private var _recorder:MicRecorder	= new MicRecorder(new WaveEncoder());	// Create a new Mic Recording Class and init with a Wav Encoder
		private var _recording:Boolean		= false;								// keep track of the recording state
		
		private var _pages:Pages			= Pages.getInstance();					// Get an instance of Pages in the app
		private var _saving:Saving			= Saving.getInstance();					// Get an instance of the Saving Singleton Class
		private var _loader:LoaderView		= LoaderView.getInstance();				// get an instance of the Loader View Singleton Class
		
		public function AudioCapture()
		{
			super();
			
			Utils.weakEvent(_recorder, RecordingEvent.RECORDING, onRecording);
			Utils.weakEvent(_recorder, Event.COMPLETE, onRecordComplete);
			Utils.weakEvent(_saving, Saving.SAVED_AUDIO, onAudioSaved);
		}
		
		
		
		// ______________________________________________
		// PUBLIC
		
		// Start recording audio
		public function toggleRecord():void
		{
			// Check the recording state
			if (!_recording)
			{
				// Start recording
				_recorder.record();
				_loader.show();
			}
			else
			{
				// Stop recording
				_recorder.stop();
				_loader.hide();
			}
			
			// toggle recording state
			_recording = !_recording;
		}
		
		
		
		
		
		// ______________________________________________
		// PROTECTED
		
		
		
		
		// ______________________________________________
		// PRIVATE
		
		// We are currently recording
		private function onRecording(e:RecordingEvent):void
		{
			// Create a message to display on the loader
			var message:String = "Tap to stop recording : " + e.time + " ms.";
			
			// Update loader message
			_loader.update(message);
		}
		
		// Save the recording to the Device
		private function onRecordComplete(e:Event):void
		{
			var ID:String	= _pages.ID;					// Get the current Page ID
			_saving.saveAudio(_recorder.output, ID);		// Save the audio using the ID as the file name
			this.dispatchEvent(new Event(AUDIO_RECORDED));	// Dispatch an event to tell the app that the audio has been recorded
		}
		
		// Audio has been saved, update the pages
		private function onAudioSaved(e:Event):void
		{
			// update the current Page data to have the audio flag to TRUE
			_pages.hasAudio = true;
			trace("onAudioSaved", _pages.hasAudio);
			
			// dispatch event to tell app to save the most up to date app data
			this.dispatchEvent(new Event(Application.SAVE_APP_DATA, true));
		}
		
		
		
		
		// ______________________________________________
		// GETTERS & SETTERS
	}
}