package uk.co.creativewax.utils
{
	import flash.display.*;
	import flash.events.*;
	import flash.geom.ColorTransform;
	import flash.geom.Matrix;
	import flash.net.URLLoader;
	import flash.net.URLRequest;
	import flash.system.ImageDecodingPolicy;
	import flash.system.LoaderContext;

	public class Utils
	{
		public static var ONE_EIGHTY_OVER_PI:Number = 180 / Math.PI;
		public static var PI_OVER_ONE_EIGHTY:Number = Math.PI / 180;
		public static var PI:Number = Math.PI;
		public static var TWO_TIMES_PI:Number = 2*Math.PI;
		public static var PI_OVER_TWO:Number = Math.PI/2;
		public static var THREE_PI_OVER_TWO:Number = (3*Math.PI)/2;
		
		private static var _cTransform:ColorTransform	= new ColorTransform();
		
		// remove children from a movie
		public static function cleanup(s:Sprite):void
		{
			var total:uint	= s.numChildren;
			for (var i:uint=0; i<total; i++) s.removeChildAt(0);
		}
		
		// check a string is boolean
		public static function parseStringBoolean(s:String):Boolean
		{
			return Boolean( s == "true" );
		}
		
		// check a string exists
		public static function doesStringExists(s:String):Boolean
		{
			if (s.length > 1 && s != "" && s != " ") return true;
			return false;
		}
		
		public static function getDateString():String
		{
			return String(new Date().getTime());
		}
		
		
		public static function weakEvent(o:Object, event:String, CBFunc:Function, remove:Boolean=false):void
		{
			if (!o)			return;
			if (!remove)	o.addEventListener(event, CBFunc, false, 0, true);
			else			o.removeEventListener(event, CBFunc);
		}
		
		
		public static function removeChild(d:DisplayObject):void
		{
			if (d == null)	return;
			if (d.parent)	d.parent.removeChild(d);
		}
		
		// work out the correct scale for this image
		public static function getScale(origW:Number, origH:Number, w:Number, h:Number, fitToBounds:Boolean=true):Number
		{
			// work out new scale
			var xScale:Number	= (1/origW)*(w);
			var yScale:Number	= (1/origH)*(h);
			var scale:Number;
			
			if (fitToBounds)	scale = Math.min(xScale, yScale);
			else				scale = Math.max(xScale, yScale);
			
			return scale;
		}
		
		
		// capture and return a cropped bitmap of this asset to the specified size passed
		public static function createBitmapToSize(asset:*, w:Number, h:Number, fitToBounds:Boolean=false):Bitmap
		{
			if (asset.width < 1 || asset.height < 1) return null;
			else if (asset.width < w && asset.height < h && asset is Bitmap) return asset;
			
			// scale this down and maintain the aspect ratio
			var scale:Number = getScale(asset.width, asset.height, w, h, fitToBounds);
			
			var matrix:Matrix	= new Matrix();
			matrix.a			= scale;
			matrix.d			= scale;
			
			// draw the asset into a temp bitmapdata object
			var bmd:BitmapData	= new BitmapData(w, h, true, 0xFFFFFF);
			bmd.draw(asset, matrix, null, null, null, true);
			
			// return new bitmap
			return new Bitmap(bmd);
		}
		
		
		// load XML file
		public static function loadXML(file:String, CBFunc:Function):void
		{			
			var urlLoader:URLLoader = new URLLoader( new URLRequest(file) );
			urlLoader.addEventListener( IOErrorEvent.IO_ERROR, Utils.onError );
			urlLoader.addEventListener( Event.COMPLETE, CBFunc );
		}
		
		
		public static function loader(file:String, CBFunc:Function=null, ProgressFunc:Function=null):Loader
		{
			var l:Loader				= new Loader();
			var context:LoaderContext	= new LoaderContext();
			context.checkPolicyFile		= true;
			context.imageDecodingPolicy	= ImageDecodingPolicy.ON_LOAD;
			
			if (CBFunc != null) l.contentLoaderInfo.addEventListener( Event.INIT, CBFunc );
			if (ProgressFunc != null) l.contentLoaderInfo.addEventListener( ProgressEvent.PROGRESS, ProgressFunc );
			l.contentLoaderInfo.addEventListener( IOErrorEvent.IO_ERROR, onError );
			l.load( new URLRequest(file), context );
			return l;
		}
		
		
		public static function onError(e:ErrorEvent):void
		{
			trace("error loading content : " + e + " : " + e.target + " : " + e.target.toString());
		}
		
		public static function tint(s:Sprite, col:uint):void
		{
			_cTransform.color				= col;
			s.transform.colorTransform		= _cTransform;
		}
		
		// return a number within a start and end range
		public static function getNumberWithinRange(num:Number, start:Number, end:Number):Number
		{
			return Math.max(start, Math.min(end, num));
		}
		
		
		// ______________________________________________
		// GETTERS & SETTERS
	}
}