package uk.co.creativewax.views
{
	import flash.display.Bitmap;
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.filesystem.File;
	
	import uk.co.creativewax.display.ImageCapture;
	import uk.co.creativewax.pages.PageData;
	import uk.co.creativewax.pages.Pages;
	import uk.co.creativewax.saving.Saving;
	import uk.co.creativewax.sound.AudioCapture;
	import uk.co.creativewax.sound.AudioPlayer;
	import uk.co.creativewax.utils.Utils;
	
	public class CaptureView extends Sprite
	{
		private var _view:CaptureViewMC				= new CaptureViewMC();			// The capture view movie from the library
		private var _imageCapture:ImageCapture		= new ImageCapture();			// Image capturing Class
		
		private var _pages:Pages					= Pages.getInstance();			// Get an instance of Pages in the app
		private var _saving:Saving					= Saving.getInstance();			// Get an instance of the Saving Singleton Class
		private var _loader:LoaderView				= LoaderView.getInstance();		// get an instance of the Loader View Singleton Class
		
		private var _audioRecorder:AudioCapture		= new AudioCapture();			// Audio capture class
		private var _audioPlayer:AudioPlayer		= new AudioPlayer();			// Audio player class
		
		private var _bmp:Bitmap;					// Current Bitmap from the Image Capture class
		
		public function CaptureView()
		{
			super();
		}
		
		
		
		// ______________________________________________
		// PUBLIC
		
		// Setup the Capture View
		public function setup():void
		{
			// Disable mouse events on the image holder
			_view.image_mc.mouseChildren	= false;
			_view.image_mc.mouseEnabled		= false;
			
			// Hide mic button until a memo has been recorded
			_view.micButton_mc.visible		= false;
			
			// Setup listeners
			Utils.weakEvent(_imageCapture, ImageCapture.IMAGE_LOADING, onImageCaptureLoading);
			Utils.weakEvent(_imageCapture, ImageCapture.IMAGE_LOADED, onImageCaptureLoaded);
			Utils.weakEvent(_audioRecorder, AudioCapture.AUDIO_RECORDED, onAudioRecorded);
			Utils.weakEvent(_saving, Saving.SAVED_IMAGE, onImageSaved);
			Utils.weakEvent(_view.photo_mc, MouseEvent.MOUSE_DOWN, onCaptureImage);
			Utils.weakEvent(_view.audio_mc, MouseEvent.MOUSE_DOWN, onCaptureAudio);
			Utils.weakEvent(_view.leftArrow_mc, MouseEvent.MOUSE_DOWN, onMoveToPrevPage);
			Utils.weakEvent(_view.rightArrow_mc, MouseEvent.MOUSE_DOWN, onMoveToNextPage);
			Utils.weakEvent(_view.micButton_mc, MouseEvent.MOUSE_DOWN, onToggleAudioPlayback);
			
			// Setup display list
			addChild(_audioRecorder);
			addChild(_view);
		}
		
		
		// Update the Capture View to reflect the current page
		public function showCurrentPageData():void
		{
			checkForImage();	// Check that the page data has an image
			checkForAudio();	// Check that the page data has a audio
			checkCurrentPage();	// Check if we are at the limits of our pages
			
			// Update the page number at the bottom of the view
			_view.page_txt.text	= String(_pages.currentPage + 1);
		}
		
		
		
		
		
		// ______________________________________________
		// PROTECTED
		
		
		
		
		// ______________________________________________
		// PRIVATE
		
		// Move to Previous / Next Page
		
		private function onMoveToPrevPage(e:MouseEvent):void	{ _pages.getPreviousPage(); }
		private function onMoveToNextPage(e:MouseEvent):void	{ _pages.getNextPage(); }
		
		
		
		/**
		 * Image methods
		 */
		
		// Capture an Image with the CameraUI
		private function onCaptureImage(e:MouseEvent):void
		{
			_imageCapture.captureImage();	// Tell Image Capture Class to start to capture an image using the CameraUI
		}
		
		// Invoked when an image has started to be loaded from the ImageCapture Class
		private function onImageCaptureLoading(e:Event):void
		{
			_loader.show("Loading Photo from Camera");	// Show standard loader
		}
		
		// Invoked when an image is returned from the ImageCapture Class
		private function onImageCaptureLoaded(e:Event):void
		{
			// If an image already exists then get rid of it
			if (_bmp)
			{
				Utils.removeChild(_bmp);	// remove from display
				_bmp = null;				// remove from memory
			}
			
			_bmp			= _imageCapture.image;	// Update latest bitmap reference
			_bmp.smoothing	= true;					// force smooth it
			showImage();							// Show the image
			saveImage();							// Save the image
			
			// hide loader
			_loader.hide();
		}
		
		// Invoked when an image has been loaded from the Device
		private function onImageLoaded(e:Event):void
		{
			_bmp			= Bitmap(e.target.content);	// Get a reference to the image
			_bmp.smoothing	= true;						// force smooth it
			showImage();								// Show the image
		}
		
		// Show the latest image
		private function showImage():void
		{
			// add to the image holder
			_view.image_mc.addChild(_bmp);
			
			// Lets show the photo bkgd
			_view.imageBkgd_mc.visible = true;
		}
		
		// Save the latest image
		private function saveImage():void
		{
			var ID:String	= _pages.ID;			// Get the current Page ID
			_saving.saveImage(_bmp.bitmapData, ID);	// Save the image using the ID as the file name
		}
		
		// Invoked when the latest image has been saved
		private function onImageSaved(e:Event):void
		{
			// update the current Page data to have the image flag to TRUE
			_pages.hasImage = true;
			
			// dispatch event to tell app to save the most up to date app data
			this.dispatchEvent(new Event(Application.SAVE_APP_DATA, true));
		}
		
		
		
		/**
		 * Audio methods
		 */
		
		// Capture an Audio
		private function onCaptureAudio(e:MouseEvent):void
		{
			_audioRecorder.toggleRecord();											// start recording the audio
			Utils.weakEvent(_loader, MouseEvent.MOUSE_DOWN, onStopAudioCapture);	// Put a listener on the loader for stopping audio capture
		}
		
		private function onStopAudioCapture(e:MouseEvent):void
		{
			_audioRecorder.toggleRecord();												// stop recording the audio
			Utils.weakEvent(_loader, MouseEvent.MOUSE_DOWN, onStopAudioCapture, true);	// Take listener off the loader for stopping audio capture
		}
		
		
		// Audio has been recorded
		private function onAudioRecorded(e:Event):void
		{
			// Show the mic button for playing back the audio
			_view.micButton_mc.visible	= true;
		}
		
		// Toggle the audio playback through the audio player
		private function onToggleAudioPlayback(e:MouseEvent):void	{ _audioPlayer.togglePlaying(); }
		
		
		
		/**
		 * Check page content
		 */
		
		// Check what the page data has an image we need to load
		private function checkForImage():void
		{
			// If an image already exists then get rid of it
			if (_bmp) Utils.removeChild(_bmp);
			
			if (_pages.hasImage)
			{
				// Resolve the path for saving the app data, this should be in the app storage directory, per iPhone guidelines.
				var file:File	= File.applicationStorageDirectory.resolvePath(_pages.ID+".png");
				
				// Load the image for the current page we are viewing
				Utils.loader(file.url, onImageLoaded);
			}
			else
			{
				// Lets hide the photo bkgd as no image is currently being shown
				_view.imageBkgd_mc.visible = false;
			}
		}
		
		// Check that the page data has a audio
		private function checkForAudio():void
		{
			if (_pages.hasAudio)	_view.micButton_mc.visible = true;	// Show mic button
			else					_view.micButton_mc.visible = false;	// Lets hide the mic button
		}
		
		// Check what page we are on
		private function checkCurrentPage():void
		{
			// Check if we are at the base position in our app
			if (_pages.currentPage <= 0)	_view.leftArrow_mc.visible = false;	// Hide previous button
			else							_view.leftArrow_mc.visible	= true;	// Show previous button
		}
		
		
		
		
		// ______________________________________________
		// GETTERS & SETTERS
	}
}