package uk.co.creativewax.views
{
	import flash.display.Bitmap;
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.filesystem.File;
	
	import uk.co.creativewax.pages.Pages;
	import uk.co.creativewax.sound.AudioPlayer;
	import uk.co.creativewax.utils.Utils;
	
	public class ViewerView extends Sprite
	{
		private var _view:ViewingViewMC			= new ViewingViewMC();			// The viewer view movie from the library
		private var _pages:Pages				= Pages.getInstance();			// Get an instance of Pages in the app
		private var _audioPlayer:AudioPlayer	= new AudioPlayer();			// Audio player class
		
		private var _bmp:Bitmap;				// Current Bitmap from the Image Capture class
		
		public function ViewerView()
		{
			super();
		}
		
		
		
		// ______________________________________________
		// PUBLIC
		
		// Setup the Viewer View
		public function setup():void
		{
			// Setup listeners
			Utils.weakEvent(_view.upArrow_mc, MouseEvent.MOUSE_DOWN, onMoveToPrevPage);
			Utils.weakEvent(_view.downArrow_mc, MouseEvent.MOUSE_DOWN, onMoveToNextPage);
			Utils.weakEvent(_view.micButton_mc, MouseEvent.MOUSE_DOWN, onToggleAudioPlayback);
			Utils.weakEvent(_pages, Pages.CURRENT_PAGE_UPDATED, checkPageContent);
			
			// Setup display list
			addChild(_view);
		}
		
		// Check the current page content
		public function checkPageContent(e:Event=null):void
		{
			// If an image already exists then get rid of it
			if (_bmp) Utils.removeChild(_bmp);
			
			// Check if we have any image
			if (_pages.hasImage)
			{
				// Resolve the path for saving the app data, this should be in the app storage directory, per iPhone guidelines.
				var file:File	= File.applicationStorageDirectory.resolvePath(_pages.ID+".png");
				
				// Load the image for the current page we are viewing
				Utils.loader(file.url, onImageLoaded);
			}
			else
			{
				// Lets hide the photo bkgd as no image is currently being shown
				_view.imageBkgd_mc.visible = false;
			}
			
			// Check if we have any audio
			if (_pages.hasAudio)	_view.micButton_mc.visible = true;	// Show mic button until a memo has been recorded
			else					_view.micButton_mc.visible = false;	// Hide mic button until a memo has been recorded
			
			// Check what page we are on
			if (_pages.currentPage == _pages.totalPages-1)	_view.downArrow_mc.visible = false;	// Hide next button
			else											_view.downArrow_mc.visible = true;	// Show next button
			
			// Update the page number at the bottom of the view
			_view.page_txt.text	= String(_pages.currentPage + 1);
		}
		
		
		
		
		
		// ______________________________________________
		// PROTECTED
		
		
		
		
		// ______________________________________________
		// PRIVATE
		
		// Invoked when an image has been loaded from the Device
		private function onImageLoaded(e:Event):void
		{
			_bmp			= Bitmap(e.target.content);	// Get a reference to the image
			_bmp.smoothing	= true;						// force smooth it
			showImage();								// Show the image
		}
		
		// Show the latest image
		private function showImage():void
		{
			// add to the image holder
			_view.image_mc.addChild(_bmp);
			
			// Lets show the photo bkgd
			_view.imageBkgd_mc.visible = true;
		}
		
		
		// Move to Previous / Next Page
		private function onMoveToPrevPage(e:MouseEvent):void	{ _pages.getPreviousPage(); }
		private function onMoveToNextPage(e:MouseEvent):void	{ _pages.getNextPage(); }
		
		// Toggle the audio playback through the audio player
		private function onToggleAudioPlayback(e:MouseEvent):void	{ _audioPlayer.togglePlaying(); }
		
		
		
		
		// ______________________________________________
		// GETTERS & SETTERS
	}
}